import { Machine } from "xstate"
import { createStore, applyMiddleware, combineReducers } from "redux"

import { createMiddleware, createReducer } from "../src"

const wait = duration =>
  new Promise((resolve, reject) => {
    setTimeout(_ => resolve(), duration)
  })

const actionMap = {
  startCountdown: (dispatch, state) =>
    wait(3000).then(() => {
      dispatch({
        type: "PED_COUNTDOWN"
      })
    })
}
const stateChart = {
  key: "light",
  initial: "green",
  states: {
    green: {
      on: {
        TIMER: "yellow"
      }
    },
    yellow: {
      on: {
        TIMER: "red"
      }
    },
    red: {
      on: {
        TIMER: "green"
      },
      states: {
        initial: "walk",
        states: {
          walk: {
            on: {
              PED_COUNTDOWN: "wait"
            },
            onEntry: ["startCountdown"]
          },
          wait: {
            on: {
              PED_COUNTDOWN: "stop"
            }
          },
          stop: {}
        }
      }
    }
  }
}

const machine = Machine(stateChart)

const store = createStore(
  combineReducers({
    machine: createReducer(machine.initialState)
  }),
  applyMiddleware(createMiddleware(machine, actionMap))
)

store.dispatch({
  type: "TIMER"
})

console.log(store.getState().machine)

store.dispatch({
  type: "TIMER"
})

console.log(store.getState().machine)
